import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class TestRPNCalculatorPublic {

    private static final double DELTA_PRECISION = 0.01;

    private static void checkStacksEqual(double[] stackSol, double[] stackToTest) {
        assertEquals("RPNStack is not correct", stackSol.length, stackToTest.length);
        for (int iter = 0; iter < stackSol.length; iter++) {
            assertEquals(String.format("RPNStack is not correct, wrong partial result in stack at position %d", iter),
                    stackSol[iter], stackToTest[iter], DELTA_PRECISION);
        }
    }

    private static void checkEquality(String[] expected, String[] toTest) {
        assertEquals("The resulting postfix order does not match the number of infix order tokens",
                expected.length, toTest.length);
        for (int iter = 0; iter < expected.length; iter++) {
            assertEquals(String.format("Postfix notation is not correct, token at position %d does not match", iter),
                    expected[iter], toTest[iter]);
        }
    }

    @Test
    public void checkRPNClass() {
        RPNCalculator cal = new RPNCalculator();
        cal.pushValue(1d);
        cal.pushValue(2d);
        cal.pushValue(3d);
        assertEquals("The method getResult() does not return the top element of the stack", 3.0, cal.getResult() , DELTA_PRECISION);
        assertEquals("The method getResult() should not remove an element from the stack", 3, cal.getStackSize());
        assertEquals("The method popResult() does not return the top element of the stack", 3.0, cal.popResult() , DELTA_PRECISION);
        assertEquals("The method popResult() should remove the retrieved element from the stack", 2, cal.getStackSize());
        checkStacksEqual(new double[] {2d, 1d}, cal.getCurrentStack());
    }

    @Test
    public void checkRPNApplyPlus() {
        RPNCalculator cal = new RPNCalculator();
        cal.pushValue(1d);
        cal.pushValue(1d);
        cal.applyOperator("+");
        assertEquals("Plus (+) is not working correctly", 2d, cal.popResult(), DELTA_PRECISION);
    }

    @Test
    public void checkRPNApplyPow() {
        RPNCalculator cal = new RPNCalculator();
        cal.pushValue(2d);
        cal.pushValue(3d);
        cal.applyOperator("^");
        assertEquals("Power (^) is not working correctly", 8d, cal.popResult(), DELTA_PRECISION);
    }

    @Test
    public void checkRPNApplyDiv() {
        RPNCalculator cal = new RPNCalculator();
        cal.pushValue(6d);
        cal.pushValue(3d);
        cal.applyOperator("/");
        assertEquals("Division (/) is not working correctly", 2d, cal.popResult(), DELTA_PRECISION);
    }

    @Test
    public void checkRPNApplyNeg() {
        RPNCalculator cal = new RPNCalculator();
        cal.pushValue(1d);
        cal.applyOperator("~");
        assertEquals("Negation (~) is not working correctly", -1d, cal.popResult(), DELTA_PRECISION);
    }

    @Test
    public void checkEval1() throws InvalidInputException {
        String expr = "1 2 + 3 - =";
        RPNCalculator cal = new RPNCalculator();
        double res = RPNCalculatorUtil.evaluatePostfixExpression(cal, RPNCalculatorUtil.parse(expr));
        assertEquals("evaluatePostfixExpression() returns a incorrect result for " + expr, 0d, res, 0.00001);
    }

    @Test
    public void checkEval2() throws InvalidInputException {
        String expr = "3.5 4 2 * 1 5 - 2 3 ^ ^ / + =";
        RPNCalculator cal = new RPNCalculator();
        double res = RPNCalculatorUtil.evaluatePostfixExpression(cal, RPNCalculatorUtil.parse(expr));
        assertEquals("evaluatePostfixExpression() returns a incorrect result for " + expr, 3.5001220703125d, res, 0.00001);
    }
    
    // Checking invalid postfix expressions in evaluatePostfixExpression
    @Test(expected = RPNException.class)
    public void checkEvalMethod3() {
    	RPNCalculator cal = new RPNCalculator();
    	String[] testCase = {"1", "1", "+", "1", "="};
        RPNCalculatorUtil.evaluatePostfixExpression(cal, testCase);
    }

    @Test
    public void checkTransform1() throws InvalidInputException {
        String[] in = RPNCalculatorUtil.parse("5 + 2 ^ 3 =");
        String[] sol =  RPNCalculatorUtil.parse("5 2 3 ^ + =");
        String[] toTest = RPNCalculatorUtil.transformInfixToPostfix(in);
        checkEquality(sol, toTest);
    }

    @Test
    public void checkTransform2() throws InvalidInputException {
        String[] in = RPNCalculatorUtil.parse("3.5 + 4 * 2 / ( 1 - 5 ) ^ 2 ^ 3 =");
        String[] sol =  RPNCalculatorUtil.parse("3.5 4 2 * 1 5 - 2 3 ^ ^ / + =");
        String[] toTest = RPNCalculatorUtil.transformInfixToPostfix(in);
        checkEquality(sol, toTest);
    }


}
